import tkinter as tk
from tkinter import messagebox
from typing import Optional


class MessageBox:
    @staticmethod
    def showinfo(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Show an info message"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.showinfo(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def showwarning(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Show a warning message"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.showwarning(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def showerror(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Show an error message"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.showerror(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def askquestion(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Ask a question"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.askquestion(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def askokcancel(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Ask if operation should proceed; return true if the answer is ok"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.askokcancel(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def askyesno(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Ask a question; return true if the answer is yes"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.askyesno(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def askyesnocancel(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Ask a question; return true if the answer is yes, None if cancelled."
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.askyesnocancel(title, message)
        root.destroy()  # 销毁主窗口
        return res

    @staticmethod
    def askretrycancel(title:Optional[str]=None, message:Optional[str]=None, **options):
        "Ask if operation should be retried; return true if the answer is yes"
        root = tk.Tk()
        root.withdraw()  # 隐藏主窗口
        res = messagebox.askretrycancel(title, message)
        root.destroy()  # 销毁主窗口
        return res



from enum import Enum


class ProcessStatus(Enum):
    UNKNOWN = ("UNKNOWN", "未知")
    TO_BE_CONFIRM = ("TO_BE_CONFIRM", "待确认")
    PENDING = ("PENDING", "待处理")
    PROCESSING = ("PROCESSING", "处理中")
    SUCCESS = ("SUCCESS", "成功")
    FAIL = ("FAIL", "失败")

    def __init__(self, code, description):
        self.code = code
        self.description = description

    @classmethod
    def get_by_desc(cls, description: str):
        for status in cls:
            if status.description == description:
                return status
        return None



from typing import Optional

from mag_tools.bean.common.data_format import DataFormat
from mag_tools.bean.common.text_format import TextFormat
from mag_tools.model.common.data_type import DataType
from mag_tools.model.common.justify_type import JustifyType


class BaseData:
    text_format: TextFormat
    data_formats: Optional[dict]

    def __init__(self):
        self.text_format = TextFormat(number_per_line=4, justify_type=JustifyType.LEFT, at_header='', decimal_places=4,
                                      decimal_places_of_zero=1)
        self.data_formats = None

    def get_data_formats(self) -> dict:
        members = vars(self)

        self.data_formats = {}
        for name, value in members.items():
            if name not in ['text_format', 'data_formats']:
                data_type = DataType.get_type(value)
                data_format = DataFormat(data_type, self.text_format.justify_type, self.text_format.decimal_places,
                                         self.text_format.decimal_places_of_zero)
                self.data_formats[name] = data_format
        return self.data_formats


class TestData(BaseData):
    def __init__(self, name: Optional[str] = None, age: Optional[int] = None, height: Optional[float] = None):
        super().__init__()

        self.name = name
        self.age = age
        self.height = height


if __name__ == '__main__':
    data = TestData(None, 12, 1)
    data_formats = data.get_data_formats()
    for name, data_format in data_formats.items():
        print(f"{name}: {data_format}")



import random
from datetime import date, datetime
from typing import Optional, List

from mag_tools.bean.common.data_format import DataFormat
from mag_tools.model.common.data_type import DataType
from mag_tools.model.common.justify_type import JustifyType
from mag_tools.utils.common.random_utils import RandomUtils
from mag_tools.utils.common.string_utils import StringUtils


class StringFormat:
    @staticmethod
    def pad_string(string:str, pad_length:int, justify_type:JustifyType=JustifyType.RIGHT)->str:
        """
        将字符串用空格补充到指定长度，空格添加在字符串前。
        参数：
        :param justify_type: 对齐方式，空格补齐
        :param string: 原始字符串
        :param pad_length: 目标长度
        :return: 补充空格后的字符串
        """
        string = str(string) if DataType.get_type(string) != DataType.STRING else string

        if len(string) >= pad_length:
            pad_length = len(string)

        padding_length = pad_length - len(string)
        if justify_type == JustifyType.RIGHT:
            return ' ' * padding_length + string
        elif justify_type == JustifyType.CENTER:
            left_padding = padding_length // 2
            right_padding = padding_length - left_padding
            return ' ' * left_padding + string + ' ' * right_padding
        else:
            return string + ' ' * padding_length

    @staticmethod
    def pad_strings(strings:List[str], pad_length:int, justify_type:JustifyType=JustifyType.RIGHT)->str:
        max_length = max(len(string) for string in strings)
        pad_length = max(max_length, pad_length)

        strings = [StringFormat.pad_string(string, pad_length, justify_type) for string in strings]
        return (' ' if max_length == pad_length else '').join(strings)

    @staticmethod
    def format_number(value: int or float, data_format: Optional[DataFormat] = None) -> str:
        real_type = DataType.get_type(value)
        if data_format is None:
            data_format = DataFormat(real_type)

        if not {data_format.data_type, real_type}.issubset({DataType.INTEGER, DataType.FLOAT}):
            return value

        if abs(value) >= 1e8 or (value != 0 and abs(value) < 1e-3):
            data_format.scientific = True

        if data_format.scientific:
            return StringUtils.float_to_scientific(value, data_format.decimal_places)

        if data_format.data_type == DataType.FLOAT:
            if value.is_integer():
                return f"{value:.{data_format.decimal_places_of_zero}f}"
            else:
                formatted_value = f"{value:.{data_format.decimal_places}f}"
                # 确保小数位末尾的零的个数符合要求
                if '.' in formatted_value:
                    integer_part, decimal_part = formatted_value.split('.')
                    decimal_part = decimal_part.rstrip('0')
                    if len(decimal_part) < data_format.decimal_places_of_zero:
                        decimal_part += '0' * (data_format.decimal_places_of_zero - len(decimal_part))
                    formatted_value = f"{integer_part}.{decimal_part}"
                return formatted_value
        elif data_format.data_type == DataType.INTEGER:
            return f"{round(value)}"

    @staticmethod
    def format(template:Optional[str])->Optional[str]:
        """
        格式化字符串，处理{}中的特定函数
        """
        if template is None:
            return None

        while '{' in template and '}' in template:
            start = template.index('{')
            end = template.index('}', start)
            expression = template[start + 1:end]

            # 解析函数名和参数
            func_name, args = StringUtils.parse_function(expression)

            # 调用相应的函数
            if func_name == 'random_text':
                result = RandomUtils.random_text(int(args[0]))
            elif func_name == 'random_string':
                result = RandomUtils.random_string(int(args[0]))
            elif func_name == 'random_chinese':
                length = int(args[0]) if args[0] else random.randint(1, 100)
                result = RandomUtils.random_chinese(length)
            elif func_name == 'random_number':
                result = RandomUtils.random_number(int(args[0]))
            elif func_name == 'random_int':
                result = str(random.randint(1, int(args[0])))
            elif func_name == 'random_date':
                year = int(args[0]) if len(args) > 0 else date.today().year
                fmt = args[1] if len(args) > 1 else "%Y%m%d"
                result = RandomUtils.random_date(year).strftime(fmt)
            elif func_name == 'today':
                fmt = args[0] if len(args) > 0 else "%Y%m%d"
                result = date.today().strftime(fmt)
            elif func_name == 'current':
                fmt = args[0] if len(args) > 0 else "%Y-%m-%d %H:%M:%S"
                result = datetime.now().strftime(fmt)
            else:
                raise ValueError(f"未知的函数: {func_name}")

            # 替换模板中的表达式
            template = template[:start] + result + template[end + 1:]

        return template



from typing import Optional

from mag_tools.bean.common.data_format import DataFormat

from mag_tools.model.common.justify_type import JustifyType
from mag_tools.model.common.data_type import DataType


class TextFormat:
    def __init__(self, number_per_line: Optional[int] = 1, justify_type: Optional[JustifyType] = JustifyType.LEFT,
                 at_header: Optional[str] = '',
                 decimal_places: Optional[int] = 2, decimal_places_of_zero: Optional[int] = 1, scientific: bool = False):
        """
        数据格式
        :param number_per_line: 每行显示的数据个数
        :param justify_type: 对齐方式
        :param at_header: 句首添加的字符串
        :param decimal_places: 小数位数
        :param decimal_places_of_zero: 小数为0时的小数位数
        """
        self.number_per_line = number_per_line
        self.justify_type = justify_type
        self.at_header = at_header
        self.decimal_places = decimal_places if decimal_places is not None else 0
        self.decimal_places_of_zero = decimal_places_of_zero if decimal_places is not None else 0
        self.scientific = scientific

    def __str__(self):
        """
        返回 TextFormat 实例的字符串表示。
        :return: TextFormat 实例的字符串表示。 """
        return f"TextFormat({', '.join(f'{key}={value}' for key, value in vars(self).items())})"

    def get_data_format(self, value):
        data_type = DataType.get_type(value)
        return DataFormat(data_type, self.justify_type,self.decimal_places, self.decimal_places_of_zero, self.scientific)
